@@ -2,7 +2,7 @@
 
 """
 Name: 'Cal3D v0.9'
-Blender: 235
+Blender: 241
 Group: 'Export'
 Tip: 'Export armature/bone/mesh/action data to the Cal3D format.'
 """
@@ -10,6 +10,7 @@ Tip: 'Export armature/bone/mesh/action data to the Cal3D format.'
 # blender2cal3D.py version 0.10
 # Copyright (C) 2003-2004 Jean-Baptiste LAMY -- jibalamy@free.fr
 # Copyright (C) 2004 Matthias Braun -- matze@braunis.de
+# Copyright (C) 2006 Loic Dachary -- loic@gnu.org
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,7 +27,7 @@ Tip: 'Export armature/bone/mesh/action data to the Cal3D format.'
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-__version__ = "0.12"
+__version__ = "0.13"
 __author__  = "Jean-Baptiste 'Jiba' Lamy"
 __email__   = "jibalamy@free.fr"
 __url__     = "Soya3d's homepage http://home.gna.org/oomadness/en/soya/"
@@ -63,7 +64,7 @@ KNOWN ISSUES
 
 NOTES
 
-It requires a very recent version of Blender (>= 2.35).
+It requires a very recent version of Blender (>= 2.41).
 
 Build a model following a few rules:
   - Use only a single armature
@@ -74,8 +75,8 @@ Build a model following a few rules:
   - Objects or animations whose names start by "_" are not exported (hidden object)
 
 It can be run in batch mode, as following :
-    blender model.blend -P blender2cal3d.py --blender2cal3d FILENAME=model.cfg EXPORT_FOR_SOYA=1
-You can pass as many parameters as you want at the end, "EXPORT_FOR_SOYA=1" is just an
+    blender model.blend -P blender2cal3d.py --blender2cal3d FILENAME=model.cfg EXPORT_FOR_GL=1
+You can pass as many parameters as you want at the end, "EXPORT_FOR_GL=1" is just an
 exemple. The parameters are the same than below.
 
 
@@ -97,6 +98,29 @@ running under gui mode the last export dir is saved into the blender registry
 and called back again.
 """
 
+#
+# Summary of changes between 0.12 and 0.13
+#
+# The changes were motivated by the API change that occured
+# between blender 2.37 and blender 2.40.
+#
+# - Use the Mathutils module instead of locally implemented vector/matrix/quat
+#   operations.
+# - Complete rewrite of the tracks export based on PoseBone and curframe
+#   instead of complex calculation based on the IPO curves
+# - Take advantage of the pre-calculated bone matrices to build the cal3d
+#   bones
+# - Rewrite EXPORT_FOR_SOYA into EXPORT_FOR_GL that transforms each matrix
+#   and vector to the GL convention instead of adding a transform node at
+#   the root of the mesh. Although a transform node is not really an issue,
+#   the undesirable side effect is that when the character is immersed in
+#   a scene using the GL convention, the developper must keep in mind that
+#   it is using a different convention. Mixing conventions within a scene
+#   is very, very error prone and confusing in the long run.
+#
+# Loic Dachary Mon Feb 27 19:11:19 CET 2006
+#
+
 # Parameters :
 
 # Filename to export to (if "", display a file selector dialog).
@@ -106,6 +130,7 @@ FILENAME = ""
 #     (http://oomadness.tuxfamily.org/en/soya).
 # (=> rotate meshes and skeletons so as X is right, Y is top and -Z is front)
 EXPORT_FOR_SOYA = 0
+EXPORT_FOR_GL = 0
 
 # Enables LODs computation. LODs computation is quite slow, and the algo is
 # surely not optimal :-(
@@ -124,9 +149,6 @@ XML = 1
 
 MESSAGES = ""
 
-# See also BASE_MATRIX below, if you want to rotate/scale/translate the model at
-# the exportation.
-
 #########################################################################################
 # Code starts here.
 # The script should be quite re-useable for writing another Blender animation exporter.
@@ -144,6 +166,7 @@ import Blender
 from Blender import Registry
 from Blender.Window import DrawProgressBar
 from Blender import Draw, BGL
+from Blender.Mathutils import *
 
 import logging 
 reload(logging)
@@ -236,258 +259,32 @@ log.addHandler(handler)
 # set this to minimum output level. eg. logging.DEBUG, logging.WARNING, logging.ERROR
 # logging.CRITICAL. logging.INFO will make little difference as these always get 
 # output'd
-log.setLevel(logging.WARNING)
-#log.setLevel(logging.DEBUG)
+#log.setLevel(logging.WARNING)
+log.setLevel(logging.DEBUG)
 
 log.info("Starting...")
 
-# transforms a blender to a cal3d quaternion notation (x,y,z,w)
-def blender2cal3dquat(q):
-  return [q.x, q.y, q.z, q.w]
+#
+# Change vector/matrix from blender convetion to OpenGL convention
+#
+ROT90X = RotationMatrix(90, 4, 'x')
+INVERT_ROT90X = RotationMatrix(90, 4, 'x').invert()
 
-def quaternion2matrix(q):
-  xx = q[0] * q[0]
-  yy = q[1] * q[1]
-  zz = q[2] * q[2]
-  xy = q[0] * q[1]
-  xz = q[0] * q[2]
-  yz = q[1] * q[2]
-  wx = q[3] * q[0]
-  wy = q[3] * q[1]
-  wz = q[3] * q[2]
-  return [[1.0 - 2.0 * (yy + zz),       2.0 * (xy + wz),       2.0 * (xz - wy), 0.0],
-          [      2.0 * (xy - wz), 1.0 - 2.0 * (xx + zz),       2.0 * (yz + wx), 0.0],
-          [      2.0 * (xz + wy),       2.0 * (yz - wx), 1.0 - 2.0 * (xx + yy), 0.0],
-          [0.0                  , 0.0                  , 0.0                  , 1.0]]
-
-def matrix2quaternion(m):
-  s = math.sqrt(abs(m[0][0] + m[1][1] + m[2][2] + m[3][3]))
-  if s == 0.0:
-    x = abs(m[2][1] - m[1][2])
-    y = abs(m[0][2] - m[2][0])
-    z = abs(m[1][0] - m[0][1])
-    if   (x >= y) and (x >= z): return 1.0, 0.0, 0.0, 0.0
-    elif (y >= x) and (y >= z): return 0.0, 1.0, 0.0, 0.0
-    else:                       return 0.0, 0.0, 1.0, 0.0
-  return quaternion_normalize([
-    -(m[2][1] - m[1][2]) / (2.0 * s),
-    -(m[0][2] - m[2][0]) / (2.0 * s),
-    -(m[1][0] - m[0][1]) / (2.0 * s),
-    0.5 * s,
-    ])
-
-def quaternion_normalize(q):
-  l = math.sqrt(q[0] * q[0] + q[1] * q[1] + q[2] * q[2] + q[3] * q[3])
-  return q[0] / l, q[1] / l, q[2] / l, q[3] / l
-
-# multiplies 2 quaternions in x,y,z,w notation
-def quaternion_multiply(q1, q2):
-  return [
-    q2[3] * q1[0] + q2[0] * q1[3] + q2[1] * q1[2] - q2[2] * q1[1],
-    q2[3] * q1[1] + q2[1] * q1[3] + q2[2] * q1[0] - q2[0] * q1[2],
-    q2[3] * q1[2] + q2[2] * q1[3] + q2[0] * q1[1] - q2[1] * q1[0],
-    q2[3] * q1[3] - q2[0] * q1[0] - q2[1] * q1[1] - q2[2] * q1[2],
-    ]
-
-def matrix_translate(m, v):
-  m[3][0] += v[0]
-  m[3][1] += v[1]
-  m[3][2] += v[2]
-  return m
-
-def matrix_multiply(b, a):
-  return [ [
-    a[0][0] * b[0][0] + a[0][1] * b[1][0] + a[0][2] * b[2][0],
-    a[0][0] * b[0][1] + a[0][1] * b[1][1] + a[0][2] * b[2][1],
-    a[0][0] * b[0][2] + a[0][1] * b[1][2] + a[0][2] * b[2][2],
-    0.0,
-    ], [
-    a[1][0] * b[0][0] + a[1][1] * b[1][0] + a[1][2] * b[2][0],
-    a[1][0] * b[0][1] + a[1][1] * b[1][1] + a[1][2] * b[2][1],
-    a[1][0] * b[0][2] + a[1][1] * b[1][2] + a[1][2] * b[2][2],
-    0.0,
-    ], [
-    a[2][0] * b[0][0] + a[2][1] * b[1][0] + a[2][2] * b[2][0],
-    a[2][0] * b[0][1] + a[2][1] * b[1][1] + a[2][2] * b[2][1],
-    a[2][0] * b[0][2] + a[2][1] * b[1][2] + a[2][2] * b[2][2],
-     0.0,
-    ], [
-    a[3][0] * b[0][0] + a[3][1] * b[1][0] + a[3][2] * b[2][0] + b[3][0],
-    a[3][0] * b[0][1] + a[3][1] * b[1][1] + a[3][2] * b[2][1] + b[3][1],
-    a[3][0] * b[0][2] + a[3][1] * b[1][2] + a[3][2] * b[2][2] + b[3][2],
-    1.0,
-    ] ]
-
-def matrix_invert(m):
-  det = (m[0][0] * (m[1][1] * m[2][2] - m[2][1] * m[1][2])
-       - m[1][0] * (m[0][1] * m[2][2] - m[2][1] * m[0][2])
-       + m[2][0] * (m[0][1] * m[1][2] - m[1][1] * m[0][2]))
-  if det == 0.0: return None
-  det = 1.0 / det
-  r = [ [
-      det * (m[1][1] * m[2][2] - m[2][1] * m[1][2]),
-    - det * (m[0][1] * m[2][2] - m[2][1] * m[0][2]),
-      det * (m[0][1] * m[1][2] - m[1][1] * m[0][2]),
-      0.0,
-    ], [
-    - det * (m[1][0] * m[2][2] - m[2][0] * m[1][2]),
-      det * (m[0][0] * m[2][2] - m[2][0] * m[0][2]),
-    - det * (m[0][0] * m[1][2] - m[1][0] * m[0][2]),
-      0.0
-    ], [
-      det * (m[1][0] * m[2][1] - m[2][0] * m[1][1]),
-    - det * (m[0][0] * m[2][1] - m[2][0] * m[0][1]),
-      det * (m[0][0] * m[1][1] - m[1][0] * m[0][1]),
-      0.0,
-    ] ]
-  r.append([
-    -(m[3][0] * r[0][0] + m[3][1] * r[1][0] + m[3][2] * r[2][0]),
-    -(m[3][0] * r[0][1] + m[3][1] * r[1][1] + m[3][2] * r[2][1]),
-    -(m[3][0] * r[0][2] + m[3][1] * r[1][2] + m[3][2] * r[2][2]),
-    1.0,
-    ])
-  return r
-
-def matrix_rotate_x(angle):
-  cos = math.cos(angle)
-  sin = math.sin(angle)
-  return [
-    [1.0,  0.0, 0.0, 0.0],
-    [0.0,  cos, sin, 0.0],
-    [0.0, -sin, cos, 0.0],
-    [0.0,  0.0, 0.0, 1.0],
-    ]
-
-def matrix_rotate_y(angle):
-  cos = math.cos(angle)
-  sin = math.sin(angle)
-  return [
-    [cos, 0.0, -sin, 0.0],
-    [0.0, 1.0,  0.0, 0.0],
-    [sin, 0.0,  cos, 0.0],
-    [0.0, 0.0,  0.0, 1.0],
-    ]
-
-def matrix_rotate_z(angle):
-  cos = math.cos(angle)
-  sin = math.sin(angle)
-  return [
-    [ cos, sin, 0.0, 0.0],
-    [-sin, cos, 0.0, 0.0],
-    [ 0.0, 0.0, 1.0, 0.0],
-    [ 0.0, 0.0, 0.0, 1.0],
-    ]
-
-def matrix_rotate(axis, angle):
-  vx  = axis[0]
-  vy  = axis[1]
-  vz  = axis[2]
-  vx2 = vx * vx
-  vy2 = vy * vy
-  vz2 = vz * vz
-  cos = math.cos(angle)
-  sin = math.sin(angle)
-  co1 = 1.0 - cos
-  return [
-    [vx2 * co1 + cos,          vx * vy * co1 + vz * sin, vz * vx * co1 - vy * sin, 0.0],
-    [vx * vy * co1 - vz * sin, vy2 * co1 + cos,          vy * vz * co1 + vx * sin, 0.0],
-    [vz * vx * co1 + vy * sin, vy * vz * co1 - vx * sin, vz2 * co1 + cos,          0.0],
-    [0.0, 0.0, 0.0, 1.0],
-    ]
-
-def matrix_scale(fx, fy, fz):
-  return [
-    [ fx, 0.0, 0.0, 0.0],
-    [0.0,  fy, 0.0, 0.0],
-    [0.0, 0.0,  fz, 0.0],
-    [0.0, 0.0, 0.0, 1.0],
-    ]
-  
-def point_by_matrix(p, m):
-  return [p[0] * m[0][0] + p[1] * m[1][0] + p[2] * m[2][0] + m[3][0],
-          p[0] * m[0][1] + p[1] * m[1][1] + p[2] * m[2][1] + m[3][1],
-          p[0] * m[0][2] + p[1] * m[1][2] + p[2] * m[2][2] + m[3][2]]
-
-def point_distance(p1, p2):
-  return math.sqrt((p2[0] - p1[0]) ** 2 + (p2[1] - p1[1]) ** 2 + (p2[2] - p1[2]) ** 2)
-
-def vector_add(v1, v2):
-  return [v1[0]+v2[0], v1[1]+v2[1], v1[2]+v2[2]]
-
-def vector_sub(v1, v2):
-  return [v1[0]-v2[0], v1[1]-v2[1], v1[2]-v2[2]]
-    
-def vector_by_matrix(p, m):
-  return [p[0] * m[0][0] + p[1] * m[1][0] + p[2] * m[2][0],
-          p[0] * m[0][1] + p[1] * m[1][1] + p[2] * m[2][1],
-          p[0] * m[0][2] + p[1] * m[1][2] + p[2] * m[2][2]]
-
-def vector_length(v):
-  return math.sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2])
-
-def vector_normalize(v):
-  l = math.sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2])
-  if l: return v[0] / l, v[1] / l, v[2] / l
-  else: return v
-
-def vector_dotproduct(v1, v2):
-  return v1[0] * v2[0] + v1[1] * v2[1] + v1[2] * v2[2]
-
-def vector_crossproduct(v1, v2):
-  return [
-    v1[1] * v2[2] - v1[2] * v2[1],
-    v1[2] * v2[0] - v1[0] * v2[2],
-    v1[0] * v2[1] - v1[1] * v2[0],
-    ]
-
-def vector_angle(v1, v2):
-  s = vector_length(v1) * vector_length(v2)
-  f = vector_dotproduct(v1, v2) / s
-  if f >=  1.0: return 0.0
-  if f <= -1.0: return math.pi / 2.0
-  return math.atan(-f / math.sqrt(1.0 - f * f)) + math.pi / 2.0
-
-def blender_bone2matrix(head, tail, roll):
-  # Convert bone rest state (defined by bone.head, bone.tail and bone.roll)
-  # to a matrix (the more standard notation).
-  # Taken from blenkernel/intern/armature.c in Blender source.
-  # See also DNA_armature_types.h:47.
-  
-  target = [0.0, 1.0, 0.0]
-  delta  = [tail[0] - head[0], tail[1] - head[1], tail[2] - head[2]]
-  nor    = vector_normalize(delta)
-  axis   = vector_crossproduct(target, nor)
-  
-  if vector_dotproduct(axis, axis) > 0.0000000000001:
-    axis    = vector_normalize(axis)
-    theta   = math.acos(vector_dotproduct(target, nor))
-    bMatrix = matrix_rotate(axis, theta)
-    
+def Vector2GL(vector):
+  if EXPORT_FOR_GL:
+    return ROT90X * vector
   else:
-    if vector_dotproduct(target, nor) > 0.0: updown =  1.0
-    else:                                    updown = -1.0
-    
-    # Quoted from Blender source : "I think this should work ..."
-    bMatrix = [
-      [updown, 0.0,    0.0, 0.0],
-      [0.0,    updown, 0.0, 0.0],
-      [0.0,    0.0,    1.0, 0.0],
-      [0.0,    0.0,    0.0, 1.0],
-      ]
+    return vector
+
+def Matrix2GL(matrix):
+  if EXPORT_FOR_GL:
+    return ROT90X * matrix * INVERT_ROT90X
+  else:
+    return matrix
   
-  rMatrix = matrix_rotate(nor, roll)
-  return matrix_multiply(rMatrix, bMatrix)
-
-
-# Hack for having the model rotated right.
-# Put in BASE_MATRIX your own rotation if you need some.
-
-BASE_MATRIX = None
-
-
 # Cal3D data structures
 
-CAL3D_VERSION = 910
+CAL3D_VERSION = 1000
 
 NEXT_MATERIAL_ID = 0
 class Material:
@@ -604,8 +401,8 @@ class SubMesh:
         if not couple_treated.has_key((a, b)):
           # The collapse factor is simply the distance between the 2 points :-(
           # This should be improved !!
-          if vector_dotproduct(a.normal, b.normal) < 0.9: continue
-          couple_collapse_factor.append((point_distance(a.loc, b.loc), a, b))
+          if DotVecs(a.normal, b.normal) < 0.9: continue
+          couple_collapse_factor.append(((a.loc - b.loc).length, a, b))
           couple_treated[a, b] = 1
       
     couple_collapse_factor.sort()
@@ -705,8 +502,8 @@ class SubMesh:
 
 class Vertex:
   def __init__(self, submesh, loc, normal):
-    self.loc    = loc
-    self.normal = normal
+    self.loc    = Vector(loc[0], loc[1], loc[2])
+    self.normal = Vector(normal[0], normal[1], normal[2])
     self.collapse_to         = None
     self.face_collapse_count = 0
     self.maps       = []
@@ -722,9 +519,11 @@ class Vertex:
     submesh.vertices.append(self)
     
   def to_cal3d(self):
+    loc = Vector2GL(self.loc)
+    normal = Vector2GL(self.normal)
     if self.collapse_to: collapse_id = self.collapse_to.id
     else:                collapse_id = -1
-    s =  struct.pack("ffffffii", self.loc[0], self.loc[1], self.loc[2], self.normal[0], self.normal[1], self.normal[2], collapse_id, self.face_collapse_count)
+    s =  struct.pack("ffffffii", loc[0], loc[1], loc[2], normal[0], normal[1], normal[2], collapse_id, self.face_collapse_count)
     s += "".join(map(Map.to_cal3d, self.maps))
     s += struct.pack("i", len(self.influences))
     s += "".join(map(Influence.to_cal3d, self.influences))
@@ -732,15 +531,17 @@ class Vertex:
     return s
   
   def to_cal3d_xml(self):
+    loc = Vector2GL(self.loc)
+    normal = Vector2GL(self.normal)
     if self.collapse_to:
       collapse_id = self.collapse_to.id
     else:
       collapse_id = -1
     s = "    <VERTEX ID=\"%i\" NUMINFLUENCES=\"%i\">\n" % \
         (self.id, len(self.influences))
-    s += "      <POS>%f %f %f</POS>\n" % (self.loc[0], self.loc[1], self.loc[2])
+    s += "      <POS>%f %f %f</POS>\n" % (loc[0], loc[1], loc[2])
     s += "      <NORM>%f %f %f</NORM>\n" % \
-         (self.normal[0], self.normal[1], self.normal[2])
+         (normal[0], normal[1], normal[2])
     if collapse_id != -1:
       s += "      <COLLAPSEID>%i</COLLAPSEID>\n" % collapse_id
       s += "      <COLLAPSECOUNT>%i</COLLAPSECOUNT>\n" % \
@@ -830,38 +631,42 @@ class Skeleton:
 BONES = {}
 
 class Bone:
-  def __init__(self, skeleton, parent, name, loc, rot):
+  def __init__(self, skeleton, parent, bone, armature_matrix):
     self.parent = parent
-    name=string.replace(name,'.','_')
-    self.name   = name
-    self.loc = loc
-    self.rot = rot
-    self.children = []
-    
-    self.matrix = self.local_matrix = matrix_translate(quaternion2matrix(rot), loc)
-    
+    self.name   = string.replace(bone.name,'.','_')
+    absolute_matrix = bone.matrix['ARMATURESPACE'] * armature_matrix
+    self.invert_matrix = Matrix(absolute_matrix).invert()
     if parent:
-      self.matrix = matrix_multiply(parent.matrix, self.matrix)
-      parent.children.append(self)
-    
-    # lloc and lrot are the bone => model space transformation (translation and rotation).
-    # They are probably specific to Cal3D.
-    m = matrix_invert(self.matrix)
-    self.lloc = m[3][0], m[3][1], m[3][2]
-    self.lrot = matrix2quaternion(m)
-    
+      matrix = absolute_matrix * self.parent.invert_matrix
+    else:
+      matrix = armature_matrix
+    self.local_matrix = matrix
+
+    self.children = []
     self.skeleton = skeleton
     self.id = skeleton.next_bone_id
+    if self.parent:
+      self.parent.children.append(self)
     skeleton.next_bone_id += 1
     skeleton.bones.append(self)
-    
-    BONES[name] = self
+    BONES[self.name] = self
     
   def to_cal3d(self):
     s =  struct.pack("i", len(self.name) + 1) + self.name + "\0"
     
-    # We need to negate quaternion W value, but why ?
-    s += struct.pack("ffffffffffffff", self.loc[0], self.loc[1], self.loc[2], self.rot[0], self.rot[1], self.rot[2], -self.rot[3], self.lloc[0], self.lloc[1], self.lloc[2], self.lrot[0], self.lrot[1], self.lrot[2], -self.lrot[3])
+    matrix = Matrix2GL(self.local_matrix)
+    lloc = matrix.translationPart()
+    lrot = matrix.toQuat()
+
+    matrix = Matrix2GL(self.invert_matrix)
+    loc = matrix.translationPart()
+    rot = matrix.toQuat()
+
+    #
+    # Negate the rotation because blender rotations are clockwise
+    # and cal3d rotations are counterclockwise
+    # 
+    s += struct.pack("ffffffffffffff", loc[0], loc[1], loc[2], rot[0], rot[1], rot[2], -rot[3], lloc[0], lloc[1], lloc[2], lrot[0], lrot[1], lrot[2], -lrot[3])
     if self.parent: s += struct.pack("i", self.parent.id)
     else:           s += struct.pack("i", -1)
     s += struct.pack("i", len(self.children))
@@ -871,15 +676,42 @@ class Bone:
   def to_cal3d_xml(self):
     s = "  <BONE ID=\"%i\" NAME=\"%s\" NUMCHILD=\"%i\">\n" % \
         (self.id, self.name, len(self.children))
-    # We need to negate quaternion W value, but why ?
+
+    #
+    # TRANSLATION and ROTATION are relative to the parent bone.
+    # They are virtually useless since the animations (.XAF .CAF)
+    # will always override them.
+    #
+    matrix = Matrix2GL(self.local_matrix)
+    translation = matrix.translationPart()
+    rotation = matrix.toQuat()
     s += "    <TRANSLATION>%f %f %f</TRANSLATION>\n" % \
-         (self.loc[0], self.loc[1], self.loc[2])
-    s += "    <ROTATION>%f %f %f %f</ROTATION>\n" % \
-         (self.rot[0], self.rot[1], self.rot[2], -self.rot[3])
+         (translation.x, translation.y, translation.z)
+    #
+    # Negate the rotation because blender rotations are clockwise
+    # and cal3d rotations are counterclockwise
+    # 
+    s += "    <ROTATION>%f %f %f -%f</ROTATION>\n" % \
+         (rotation.x, rotation.y, rotation.z, rotation.w)
+
+    #
+    # LOCALTRANSLATION and LOCALROTATION are the invert of the cumulated
+    # TRANSLATION and ROTATION (see above). It is used to calculate the
+    # delta between an animated bone and the original non animated bone.
+    # This delta will be applied to the influenced vertexes. 
+    #
+    matrix = Matrix2GL(self.invert_matrix)
+    translation = matrix.translationPart()
+    rotation = matrix.toQuat()
     s += "    <LOCALTRANSLATION>%f %f %f</LOCALTRANSLATION>\n" % \
-         (self.lloc[0], self.lloc[1], self.lloc[2])
-    s += "    <LOCALROTATION>%f %f %f %f</LOCALROTATION>\n" % \
-         (self.lrot[0], self.lrot[1], self.lrot[2], -self.lrot[3])
+         (translation.x, translation.y, translation.z)
+    #
+    # Negate the rotation because blender rotations are clockwise
+    # and cal3d rotations are counterclockwise
+    # 
+    s += "    <LOCALROTATION>%f %f %f -%f</LOCALROTATION>\n" % \
+         (rotation.x, rotation.y, rotation.z, rotation.w)
+
     if self.parent:
       s += "    <PARENTID>%i</PARENTID>\n" % self.parent.id
     else:
@@ -940,26 +772,28 @@ class KeyFrame:
     track.keyframes.append(self)
     
   def to_cal3d(self):
-    # We need to negate quaternion W value, but why ?
-    return struct.pack("ffffffff", self.time, self.loc[0], self.loc[1], self.loc[2], self.rot[0], self.rot[1], self.rot[2], -self.rot[3])
+    #
+    # Negate the rotation because blender rotations are clockwise
+    # and cal3d rotations are counterclockwise
+    # 
+    return struct.pack("ffffffff", self.time, self.loc.x, self.loc.y, self.loc.z, self.rot.x, self.rot.y, self.rot.z, -self.rot.w)
   
   def to_cal3d_xml(self):
     s = "    <KEYFRAME TIME=\"%f\">\n" % self.time
     s += "      <TRANSLATION>%f %f %f</TRANSLATION>\n" % \
-         (self.loc[0], self.loc[1], self.loc[2])
-    # We need to negate quaternion W value, but why ?
-    s += "      <ROTATION>%f %f %f %f</ROTATION>\n" % \
-         (self.rot[0], self.rot[1], self.rot[2], -self.rot[3])
+         (self.loc.x, self.loc.y, self.loc.z)
+    #
+    # Negate the rotation because blender rotations are clockwise
+    # and cal3d rotations are counterclockwise
+    # 
+    s += "      <ROTATION>%f %f %f -%f</ROTATION>\n" % \
+         (self.rot.x, self.rot.y, self.rot.z, self.rot.w)
     s += "    </KEYFRAME>\n"
     return s                                                      
   
 def export(filename):
   global MESSAGES
-  
-  if EXPORT_FOR_SOYA:
-    global BASE_MATRIX
-    BASE_MATRIX = matrix_rotate_x(-math.pi / 2.0)
-    
+
   # Get the scene
   scene = Blender.Scene.getCurrent()
   
@@ -970,76 +804,37 @@ def export(filename):
   skeleton = Skeleton()
 
   foundarmature = False
+  armature = None
   for obj in Blender.Object.Get():
     data = obj.getData()
     if type(data) is not Blender.Types.ArmatureType:
       continue
-    
+
+    armature = obj
+
     if foundarmature == True:
       log.error("Found multiple armatures! '" + obj.getName() + "' ignored.\n")
       continue
 
     foundarmature = True
     matrix = obj.getMatrix()
-    if BASE_MATRIX:
-      matrix = matrix_multiply(BASE_MATRIX, matrix)
     
     def treat_bone(b, parent = None):
-      head = b.getHead()
-      tail = b.getTail()
-      
-      # Turns the Blender's head-tail-roll notation into a quaternion
-      quat = matrix2quaternion(blender_bone2matrix(head, tail, b.getRoll()))
-      
       if parent:
-        # Compute the translation from the parent bone's head to the child
-        # bone's head, in the parent bone coordinate system.
-        # The translation is parent_tail - parent_head + child_head,
-        # but parent_tail and parent_head must be converted from the parent's parent
-        # system coordinate into the parent system coordinate.
-        
-        parent_invert_transform = matrix_invert(quaternion2matrix(parent.rot))
-        parent_head = vector_by_matrix(parent.head, parent_invert_transform)
-        parent_tail = vector_by_matrix(parent.tail, parent_invert_transform)
+        log.debug("Parented Bone: %s",b.name)
 
-        ploc = vector_add(head, b.getLoc())
-        parentheadtotail = vector_sub(parent_tail, parent_head)
-        # hmm this should be handled by the IPos, but isn't for non-animated
-        # bones which are transformed in the pose mode...
-        #loc = vector_add(ploc, parentheadtotail)
-        #rot = quaternion_multiply(blender2cal3dquat(b.getQuat()), quat)
-        loc = parentheadtotail
-        rot = quat
-        
-        log.debug("Parented Bone: %s",b.getName())
-
-        bone = Bone(skeleton, parent, b.getName(), loc, rot)
+        bone = Bone(skeleton, parent, b, matrix)
       else:
-        # Apply the armature's matrix to the root bones
-        head = point_by_matrix(head, matrix)
-        tail = point_by_matrix(tail, matrix)
-        quat = matrix2quaternion(matrix_multiply(matrix, quaternion2matrix(quat))) # Probably not optimal
-        
-        # loc = vector_add(head, b.getLoc())
-        # rot = quaternion_multiply(blender2cal3dquat(b.getQuat()), quat)
-        loc = head
-        rot = quat
-        
-        log.debug("Non Parented Bone: %s",b.getName())
+        bone = Bone(skeleton, None, b, matrix)
 
-        # Here, the translation is simply the head vector
-        bone = Bone(skeleton, None, b.getName(), loc, rot)
-        
-      bone.head = head
-      bone.tail = tail
-      
-      for child in b.getChildren():
-        treat_bone(child, bone)
+      if b.hasChildren():
+        for child in b.children:
+          treat_bone(child, bone)
      
     foundroot = False
-    for b in data.getBones():
+    for b in data.bones.values():
       # child bones are handled in treat_bone
-      if b.getParent() != None:
+      if b.parent != None:
         continue
       if foundroot == True:
         log.warning("Warning: Found multiple root-bones, this may not be supported in cal3d.")
@@ -1067,8 +862,6 @@ def export(filename):
       meshes.append(mesh)
       
       matrix = obj.getMatrix()
-      if BASE_MATRIX:
-        matrix = matrix_multiply(BASE_MATRIX, matrix)
         
       faces = data.faces
       while faces:
@@ -1094,18 +887,18 @@ def export(filename):
                 log.error("You have faces with less that three verticies!")
                 continue
 
-              normal = vector_normalize(vector_by_matrix(vector_crossproduct(
-                [p3[0] - p2[0], p3[1] - p2[1], p3[2] - p2[2]],
-                [p1[0] - p2[0], p1[1] - p2[1], p1[2] - p2[2]],
-                ), matrix))
-              
+              normal = CrossVecs(
+                Vector(p3[0] - p2[0], p3[1] - p2[1], p3[2] - p2[2]),
+                Vector(p1[0] - p2[0], p1[1] - p2[1], p1[2] - p2[2]),
+                ).resize4D() * matrix
+              normal = Vector(normal[0], normal[1], normal[2]).normalize()
+
             face_vertices = []
             for i in range(len(face.v)):
               vertex = vertices.get(face.v[i].index)
               if not vertex:
-                coord    = point_by_matrix (face.v[i].co, matrix)
-                if face.smooth:
-                  normal = vector_normalize(vector_by_matrix(face.v[i].no, matrix))
+                coord    = Vector(face.v[i].co).resize4D() * matrix
+                normal   = face.v[i].no
                 vertex   = vertices[face.v[i].index] = Vertex(submesh, coord, normal)
                 
                 influences = data.getVertexInfluences(face.v[i].index)
@@ -1124,7 +917,8 @@ def export(filename):
                 # sum of influences is not always 1.0 in Blender ?!?!
                 sum = 0.0
                 for bone_name, weight in influences:
-                  sum += weight
+                  if bone_name in BONES:
+                    sum += weight
                 
                 for bone_name, weight in influences:
                   bone_name=string.replace(bone_name,'.','_')
@@ -1144,7 +938,6 @@ def export(filename):
                 # We cannot share vertex for non-smooth faces, since Cal3D does not
                 # support vertex sharing for 2 vertices with different normals.
                 # => we must clone the vertex.
-                
                 old_vertex = vertex
                 vertex = Vertex(submesh, vertex.loc, normal)
                 vertex.cloned_from = old_vertex
@@ -1189,92 +982,63 @@ def export(filename):
 
   ANIMATIONS = {}
 
-  for a in Blender.Armature.NLA.GetActions().iteritems():
-    animation_name = a[0]
-    
-    log.debug( "Animation: %s",animation_name)
-    
-    animation = Animation(animation_name)
-    animation.duration = 0.0
+  for (action_name, action) in Blender.Armature.NLA.GetActions().iteritems():
+    animation = Animation(action_name)
 
-    for b in a[1].getAllChannelIpos().iteritems():
-      bone_name = string.replace(b[0],'.','_')
-      if bone_name not in BONES:
-        log.error("No Bone '" + bone_name + "' defined (from Animation '" + animation_name + "' ?!?\n")
-        continue                                            
+    action.setActive(armature)
 
+    name2ipo = {}
+    frames = []
+    for (ipo_name, ipo) in action.getAllChannelIpos().iteritems():
+      has_frames = False
+      for curve in ipo.getCurves():
+        for point in curve.getPoints():
+          has_frames = True
+          frame = point.pt[0]
+          if frame not in frames:
+            frames.append(int(frame))
+      if has_frames:
+        name2ipo[ipo_name] = ipo
+    frames.sort()
+
+    if not frames:
+      continue
+
+    animation.duration = frames[-1]
+
+    for (ipo_name, ipo) in name2ipo.iteritems():
+
+      bone_name = string.replace(ipo_name,'.','_')
       bone = BONES[bone_name]
-
       track = Track(animation, bone)
       track.finished = 0
       animation.tracks[bone_name] = track
 
-      ipo = b[1]
-      
-      times = []
-      
-      # SideNote: MatzeB: Ipo.getCurve(curvename) is broken in blender 2.33 and
-      # below if the Ipo comes from an Action, so only use Ipo.getCurves()!
-      # also blender upto 2.33a had a bug where IpoCurve.evaluate was not
-      # exposed to the python interface :-/
-      
-      #run 1: we need to find all time values where we need to produce keyframes
-      for curve in ipo.getCurves():
-        curve_name = curve.getName()
+      blender_bone = armature.getPose().bones[ipo_name]
+      for curframe in frames:
+        scene.getRenderingContext().currentFrame(curframe)
+        scene.update(1)
+        matrix = blender_bone.quat.toMatrix()
+        matrix.resize4x4()
+        matrix[3] = Vector(*blender_bone.loc).resize4D()
+        if blender_bone.size != Vector(1, 1, 1):
+          log.error("Action " + action_name + ": Bone " + ipo_name + " resized " + str(blender_bone.size) + " at frame " + str(curframe) + " not supported and ignored")
+#        scaleMatrix = ScaleMatrix(blender_bone.size[0], 4, Vector(1, 0, 0)) * ScaleMatrix(blender_bone.size[1], 4, Vector(0, 1, 0)) * ScaleMatrix(blender_bone.size[2], 4, Vector(0, 0, 1))
+#        print bone_name + "\nscale Matrix\n" + str(scaleMatrix)
+#        matrix *= scaleMatrix
+ #       print bone_name + "\ndelta Matrix\n" + str(matrix)
+        matrix = Matrix2GL(matrix * bone.local_matrix)
+#        print bone_name + "\ncal3d Matrix\n" + str(matrix)
+        #
+        # Assume 25 fps since there is currently (blender 2.41) no way to
+        # query the fps of the timeline
+        #
+        KeyFrame(track, (curframe-1) / 25.0, matrix.translationPart(), matrix.toQuat())
 
-        if curve_name not in ["QuatW", "QuatX", "QuatY", "QuatZ", "LocX", "LocY", "LocZ"]:
-          log.error("Curve type %s not supported in Action '%s' Bone '%s'.\n"\
-                    % (curve_name, animation_name, bone_name))
-        
-        for p in curve.getPoints():
-          time = p.getPoints() [0]
-          if time not in times:
-            times.append(time)
+    if animation.duration > 0:
+      animation.duration /= 25.0
+      ANIMATIONS[action_name] = animation
       
-      times.sort()
-
-      # run2: now create keyframes
-      for time in times:
-        cal3dtime = (time-1) / 25.0 # assume 25FPS by default
-        if cal3dtime > animation.duration:
-          animation.duration = cal3dtime
-        trans = [0, 0, 0]
-        quat  = [0, 0, 0, 0]
-        
-        for curve in ipo.getCurves():
-          val = curve.evaluate(time)
-          if curve.getName() == "LocX": trans[0] = val
-          if curve.getName() == "LocY": trans[1] = val
-          if curve.getName() == "LocZ": trans[2] = val
-          if curve.getName() == "QuatW": quat[3] = val
-          if curve.getName() == "QuatX": quat[0] = val
-          if curve.getName() == "QuatY": quat[1] = val
-          if curve.getName() == "QuatZ": quat[2] = val
-          log.debug('Curve: %s' % curve.getName())
-
-        if quat==[0,0,0,0]:
-          log.critical('You are using just Loc keys. You must use LocRot keys instead.')
-          continue
-        
-        transt = vector_by_matrix(trans, bone.local_matrix)
-          
-        loc = vector_add(bone.loc, transt)
-        rot = quaternion_multiply(quat, bone.rot)
-        
-        try:
-          rot = quaternion_normalize(rot)
-        except:
-          import traceback
-          log.error('quaternion_normalize failed')
-
-        KeyFrame(track, cal3dtime, loc, rot)
-        
-    if animation.duration <= 0:
-      log.warning("Ignoring Animation '" + animation_name + \
-                  "': duration is 0.")
-      continue
-    ANIMATIONS[animation_name] = animation
-    
   # Save all data
   if filename.endswith(".cfg"):
     filename = os.path.splitext(filename)[0]
@@ -1299,6 +1063,7 @@ def export(filename):
     print >> cfg
     
   filename = BASENAME + "." + FORMAT_PREFIX + "sf"
+  log.debug("FILENAME %s" % filename)
   open(os.path.join(DIRNAME, filename), "wb").write(encode(skeleton))
   print >> cfg, "skeleton=%s" % filename
   print >> cfg
@@ -1425,6 +1190,10 @@ def get_from_registry():
   else:
     return ''
 
+
+if EXPORT_FOR_SOYA:
+  EXPORT_FOR_GL = EXPORT_FOR_SOYA
+
 # Check for batch mode
 if "--blender2cal3d" in sys.argv:
   args = sys.argv[sys.argv.index("--blender2cal3d") + 1:]
