@@ -1,5 +1,6 @@
 /*
  *
+ * Copyright (C) 2007 Loic Dachary <loic@dachary.org>
  * Copyright (C) 2005, 2006 Mekensleep
  *
  *	Mekensleep
@@ -286,7 +287,7 @@ bool GetNumber(int& result,char* number)
 {
   char* endptr;
   errno = 0;
-  int val = (int)strtol(number,&endptr,10);
+  long val = strtol(number,&endptr,10);
   if ((errno == ERANGE && (val == LONG_MAX || val == LONG_MIN))
       || (errno != 0 && val == 0)) {
     return false;
