@@ -1,5 +1,6 @@
 /* -*- c++ -*-
 *
+* Copyright (C) 2007 Loic Dachary <loic@dachary.org>
 * Copyright (C) 2004, 2006 Mekensleep
 *
 * Mekensleep
@@ -238,7 +239,7 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
     mon = specialOptions->getMonitor();
 
   {
-    char* xpath = "/cal3d";
+    const char* xpath = "/cal3d";
     xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression((xmlChar*)xpath, xpathContext);
     if(xpathObj == NULL) 
       osg::notify(osg::FATAL) << "LoadXML: eval " << xpath << " file in path " << path << std::endl;
@@ -279,7 +280,7 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
   }
 
   {
-    char* xpath = "/cal3d/skeleton/@name";
+    const char* xpath = "/cal3d/skeleton/@name";
     xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression((xmlChar*)xpath, xpathContext);
     if(xpathObj == NULL) 
       osg::notify(osg::FATAL) << "LoadXML: eval " << xpath << " file in path " << path << std::endl;
@@ -315,7 +316,7 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
   }
 
   {
-    char* xpath = "/cal3d/library/mesh";
+    const char* xpath = "/cal3d/library/mesh";
     xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression((xmlChar*)xpath, xpathContext);
     if(xpathObj == NULL) 
       osg::notify(osg::FATAL) <<   "LoadXML: eval " << xpath << " file in path " << path << std::endl;
@@ -379,7 +380,7 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
   }
 
   {
-    char* xpath = "/cal3d/library/material";
+    const char* xpath = "/cal3d/library/material";
     xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression((xmlChar*)xpath, xpathContext);
     if(xpathObj == NULL) {
       osg::notify(osg::FATAL) << "eval " << xpath << " file in " << path << std::endl;
@@ -470,7 +471,7 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
   }
 
   {
-    char* xpath = "/cal3d/library/animation";
+    const char* xpath = "/cal3d/library/animation";
     xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression((xmlChar*)xpath, xpathContext);
     if(xpathObj == NULL) {
       osg::notify(osg::FATAL) << "eval " << xpath << " file in " << path << std::endl;
@@ -557,7 +558,7 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
 	//MessageBox(NULL, str, "hop", MB_OK); 
 
   {
-    char* xpath = "/cal3d/common/mesh";
+    const char* xpath = "/cal3d/common/mesh";
     xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression((xmlChar*)xpath, xpathContext);
     if(xpathObj == NULL) {
       osg::notify(osg::FATAL) << "eval " << xpath << " file in " << path << std::endl;
@@ -607,7 +608,7 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
   }
 
   {
-    char* xpath = "/cal3d/parameters/parameter";
+    const char* xpath = "/cal3d/parameters/parameter";
     xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression((xmlChar*)xpath, xpathContext);
     if(xpathObj == NULL) {
       osg::notify(osg::FATAL) << "eval " << xpath << " file in " << path << std::endl;
@@ -814,7 +815,7 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
 
 
   {
-    char* xpath = "/cal3d/layers/layer";
+    const char* xpath = "/cal3d/layers/layer";
     xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression((xmlChar*)xpath, xpathContext);
     if(xpathObj == NULL) {
       osg::notify(osg::FATAL) << "eval " << xpath << " file in " << path << std::endl;
@@ -897,7 +898,7 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
 
 
   {
-    char* xpath = "/cal3d/uislots";
+    const char* xpath = "/cal3d/uislots";
     xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression((xmlChar*)xpath, xpathContext);
     if(xpathObj == NULL) {
       osg::notify(osg::FATAL) << "eval " << xpath << " file in " << path << std::endl;
