@@ -1,5 +1,6 @@
 /*
 *
+* Copyright (C) 2007 Loic Dachary <loic@dachary.org>
 * Copyright (C) 2004, 2005, 2006 Mekensleep
 *
 * Mekensleep
@@ -1724,7 +1725,11 @@ void TextureLayersFlatten::MultiplyRenderTechnic::setup(osg::Group *_on)
 
 // TextureLayersFlatten::MyGeometry
 
+#if OSG_VERSION_MAJOR != 2
 void TextureLayersFlatten::MyGeometry::drawImplementation(osg::State &_state) const
+#else // OSG_VERSION_MAJOR != 2
+void TextureLayersFlatten::MyGeometry::drawImplementation(osg::RenderInfo &_renderInfo) const
+#endif // OSG_VERSION_MAJOR != 2
 {
   int i;
 
@@ -1753,14 +1758,15 @@ void TextureLayersFlatten::MyGeometry::drawImplementation(osg::State &_state) co
     tex->setFilter(osg::Texture::MAG_FILTER, osg::Texture::NEAREST);
   }
 
+#if OSG_VERSION_MAJOR != 2
 	Geometry::drawImplementation( _state );
+#else // OSG_VERSION_MAJOR != 2
+	Geometry::drawImplementation( _renderInfo );
+#endif // OSG_VERSION_MAJOR != 2
 
   if (target_.valid()) {
     osg::Texture2D *tgt = (osg::Texture2D*) target_.get();
 
-//    _state.setActiveTextureUnit(0);
-  //  _state.haveAppliedTextureAttribute(0, tgt);
-
     int w = tgt->getTextureWidth();
     int h = tgt->getTextureHeight();
     if (!w) {
@@ -1768,10 +1774,12 @@ void TextureLayersFlatten::MyGeometry::drawImplementation(osg::State &_state) co
       h = tgt->getImage()->t();
     }
 
+#if OSG_VERSION_MAJOR != 2
     tgt->copyTexImage2D(_state, 0, 0, w, h);
+#else // OSG_VERSION_MAJOR != 2
+    tgt->copyTexImage2D(*_renderInfo.getState(), 0, 0, w, h);
+#endif // OSG_VERSION_MAJOR != 2
 
-//		int t =  _state.getActiveTextureUnit();
-  //  _state.haveAppliedTextureAttribute(t, target_.get() );
 	}
 
   for (i = 0; i < nbTexs; i++) {
